var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
/**
 * YYG 事件类型
 */
var ADEvent = /** @class */ (function () {
    function ADEvent() {
    }
    /**
     * 广告开始播放
     */
    ADEvent.AD_STARTED = "AD_STARTED";
    /**
     * 广告加载完成
     */
    ADEvent.AD_LOADED = "AD_LOADED";
    /**
     * 广告点击
     */
    ADEvent.AD_CLICK = "AD_CLICK";
    /**
     * 广告播放完成
     */
    ADEvent.AD_COMPLETE = "AD_COMPLETE";
    /**
     * 广告点击跳过
     */
    ADEvent.AD_SKIPPED = "AD_SKIPPED";
    /**
     * 游戏内广告不可用
     */
    ADEvent.AD_INGAME_DISABLED = "AD_INGAME_DISABLED";
    /**
     * 广告请求过快
     */
    ADEvent.AD_REQUEST_TOO_SOON = "AD_REQUEST_TOO_SOON";
    /**
     * 广告错误
     */
    ADEvent.AD_ERROR = "AD_ERROR";
    /**
     * SDK初始化成功
     */
    ADEvent.YYGSDK_INITIALIZED = "SDK_INITIALIZED";
    /**
     * SDK初始化成功
     */
    ADEvent.YYGSDK_NOT_INITIALIZED = "YYGSDK_NOT_INITIALIZED";
    /**
     * 服务器请求参数超时
     */
    ADEvent.SEVRVER_OPTIONS_TIMEOUT = "SEVRVER_OPTIONS_TIMEOUT";
    return ADEvent;
}());
/**
 * <p><code>EventHandler</code> 是事件处理器类。</p>
 * <p>推荐使用 EventHandler.create() 方法从对象池创建，减少对象创建消耗。
 * 创建的 Handler 对象不再使用后，可以使用 Handler.recover() 将其回收到对象池，回收后不要再使用此对象，否则会导致不可预料的错误。</p>
 */
var EventHandler = /** @class */ (function () {
    /**
     * 根据指定的属性值，创建一个 <code>Handler</code> 类的实例。
     * @param	caller 执行域。
     * @param	method 处理函数。
     * @param	args 函数参数。
     * @param	once 是否只执行一次。
     */
    function EventHandler(caller, method, args, once) {
        this.once = false;
        this._id = 0;
        this.setTo(caller, method, args, once);
    }
    /**
     * 设置此对象的指定属性值。
     * @param	caller 执行域(this)。
     * @param	method 回调方法。
     * @param	args 携带的参数。
     * @param	once 是否只执行一次，如果为true，执行后执行recover()进行回收。
     * @return  返回 handler 本身。
     */
    EventHandler.prototype.setTo = function (caller, method, args, once) {
        this._id = EventHandler._gid++;
        this.caller = caller;
        this.method = method;
        this.args = args;
        this.once = once;
        return this;
    };
    /**
     * 执行处理器。
     */
    EventHandler.prototype.run = function () {
        if (this.method == null)
            return null;
        var id = this._id;
        var result = this.method.apply(this.caller, this.args);
        this._id === id && this.once && this.recover();
        return result;
    };
    /**
     * 执行处理器，携带额外数据。
     * @param	data 附加的回调数据，可以是单数据或者Array(作为多参)。
     */
    EventHandler.prototype.runWith = function (data) {
        if (this.method == null)
            return null;
        var id = this._id;
        if (data == null)
            var result = this.method.apply(this.caller, this.args);
        else if (!this.args && !data.unshift)
            result = this.method.call(this.caller, data);
        else if (this.args)
            result = this.method.apply(this.caller, this.args.concat(data));
        else
            result = this.method.apply(this.caller, data);
        this._id === id && this.once && this.recover();
        return result;
    };
    EventHandler.prototype.clear = function () {
        this.caller = null;
        this.method = null;
        this.args = null;
        return this;
    };
    /**
     * 清理对象引用。
     */
    EventHandler.prototype.recover = function () {
        if (this._id > 0) {
            this._id = 0;
            EventHandler._pool.push(this.clear());
        }
    };
    /**
     * 从对象池内创建一个Handler，默认会执行一次并立即回收，如果不需要自动回收，设置once参数为false。
     * @param	caller 执行域(this)。
     * @param	method 回调方法。
     * @param	args 携带的参数。
     * @param	once 是否只执行一次，如果为true，回调后执行recover()进行回收，默认为true。
     * @return  返回创建的handler实例。
     */
    EventHandler.create = function (caller, method, args, once) {
        if (args === void 0) { args = null; }
        if (once === void 0) { once = true; }
        if (EventHandler._pool.length)
            return EventHandler._pool.pop().setTo(caller, method, args, once);
        return new EventHandler(caller, method, args, once);
    };
    EventHandler._pool = [];
    EventHandler._gid = 1;
    return EventHandler;
}());
var EventDispatcher = /** @class */ (function () {
    function EventDispatcher() {
    }
    /**
     * 检查 EventDispatcher 对象是否为特定事件类型注册了任何侦听器。
     * @param	type 事件的类型。
     * @return 如果指定类型的侦听器已注册，则值为 true；否则，值为 false。
     */
    EventDispatcher.prototype.hasListener = function (type) {
        var listener = this._events && this._events[type];
        return !!listener;
    };
    /**
     * 派发事件。
     * @param type	事件类型。
     * @param data	（可选）回调数据。<b>注意：</b>如果是需要传递多个参数 p1,p2,p3,...可以使用数组结构如：[p1,p2,p3,...] ；如果需要回调单个参数 p ，且 p 是一个数组，则需要使用结构如：[p]，其他的单个参数 p ，可以直接传入参数 p。
     * @return 此事件类型是否有侦听者，如果有侦听者则值为 true，否则值为 false。
     */
    EventDispatcher.prototype.event = function (type, data) {
        if (!this._events || !this._events[type])
            return false;
        var listeners = this._events[type];
        if (listeners.run) {
            if (listeners.once)
                delete this._events[type];
            data != null ? listeners.runWith(data) : listeners.run();
        }
        else {
            for (var i = 0, n = listeners.length; i < n; i++) {
                var listener = listeners[i];
                if (listener) {
                    (data != null) ? listener.runWith(data) : listener.run();
                }
                if (!listener || listener.once) {
                    listeners.splice(i, 1);
                    i--;
                    n--;
                }
            }
            if (listeners.length === 0 && this._events)
                delete this._events[type];
        }
        return true;
    };
    /**
     * 使用 EventDispatcher 对象注册指定类型的事件侦听器对象，以使侦听器能够接收事件通知。
     * @param type		事件的类型。
     * @param caller	事件侦听函数的执行域。
     * @param listener	事件侦听函数。
     * @param args		（可选）事件侦听函数的回调参数。
     * @return 此 EventDispatcher 对象。
     */
    EventDispatcher.prototype.on = function (type, caller, listener, args) {
        return this._createListener(type, caller, listener, args, false);
    };
    /**
     * 使用 EventDispatcher 对象注册指定类型的事件侦听器对象，以使侦听器能够接收事件通知，此侦听事件响应一次后自动移除。
     * @param type		事件的类型。
     * @param caller	事件侦听函数的执行域。
     * @param listener	事件侦听函数。
     * @param args		（可选）事件侦听函数的回调参数。
     * @return 此 EventDispatcher 对象。
     */
    EventDispatcher.prototype.once = function (type, caller, listener, args) {
        return this._createListener(type, caller, listener, args, true);
    };
    /**
     * 从 EventDispatcher 对象中删除侦听器。
     * @param type		事件的类型。
     * @param caller	事件侦听函数的执行域。
     * @param listener	事件侦听函数。
     * @param onceOnly	（可选）如果值为 true ,则只移除通过 once 方法添加的侦听器。
     * @return 此 EventDispatcher 对象。
     */
    EventDispatcher.prototype.off = function (type, caller, listener, onceOnly) {
        if (!this._events || !this._events[type])
            return this;
        var listeners = this._events[type];
        if (listeners != null) {
            if (listeners.run) {
                if ((!caller || listeners.caller === caller) && (listener == null || listeners.method === listener) && (!onceOnly || listeners.once)) {
                    delete this._events[type];
                    listeners.recover();
                }
            }
            else {
                var count = 0;
                for (var i = 0, n = listeners.length; i < n; i++) {
                    var item = listeners[i];
                    if (!item) {
                        count++;
                        continue;
                    }
                    if (item && (!caller || item.caller === caller) && (listener == null || item.method === listener) && (!onceOnly || item.once)) {
                        count++;
                        listeners[i] = null;
                        item.recover();
                    }
                }
                if (count === n)
                    delete this._events[type];
            }
        }
        return this;
    };
    /**
     * 从 EventDispatcher 对象中删除指定事件类型的所有侦听器。
     * @param type	（可选）事件类型，如果值为 null，则移除本对象所有类型的侦听器。
     * @return 此 EventDispatcher 对象。
     */
    EventDispatcher.prototype.offAll = function (type) {
        var events = this._events;
        if (!events)
            return this;
        if (type) {
            this._recoverHandlers(events[type]);
            delete events[type];
        }
        else {
            for (var name in events) {
                this._recoverHandlers(events[name]);
            }
            this._events = null;
        }
        return this;
    };
    /**
     * 移除caller为target的所有事件监听
     * @param caller caller对象
     */
    EventDispatcher.prototype.offAllCaller = function (caller) {
        if (caller && this._events) {
            for (var name in this._events) {
                this.off(name, caller, null);
            }
        }
        return this;
    };
    EventDispatcher.prototype._recoverHandlers = function (arr) {
        if (!arr)
            return;
        if (arr.run) {
            arr.recover();
        }
        else {
            for (var i = arr.length - 1; i > -1; i--) {
                if (arr[i]) {
                    arr[i].recover();
                    arr[i] = null;
                }
            }
        }
    };
    EventDispatcher.prototype._createListener = function (type, caller, listener, args, once, offBefore) {
        offBefore && this.off(type, caller, listener, once);
        var handler = EventHandler.create(caller || this, listener, args, once);
        this._events || (this._events = {});
        var events = this._events;
        if (!events[type])
            events[type] = handler;
        else {
            if (!events[type].run)
                events[type].push(handler);
            else
                events[type] = [events[type], handler];
        }
        return this;
    };
    return EventDispatcher;
}());
/**
 * AD 为广告管理器
 */
var platform = /** @class */ (function (_super) {
    __extends(platform, _super);
    function platform() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.initialized_ = false;
        return _this;
    }
    platform.prototype.startup = function (gamedistributionId, onComplete) {
        var _this = this;
        if (gamedistributionId === void 0) { gamedistributionId = ""; }
        if (!this.initialized_) {
            var script = document.getElementById("gamedistribution-jssdk");
            if (!script) {
                try {
                    window["GD_OPTIONS"] = {
                        gameId: gamedistributionId,
                        onEvent: this.onEvent.bind(this)
                    };
                    script = document.createElement("script");
                    script.onload = function () { console.log("aaaa"),_this.initialized_ = true;  onComplete && onComplete();};
                    script.type   = "text/javascript";
                    script.async  = true;
                    script.src    = "https://html5.api.gamedistribution.com/main.min.js";
                    script.id     = "gamedistribution-jssdk";
                    document.head.appendChild(script);
                }
                catch (error) {
                    console.error("aaaa")
                }
            }
        }
    };
    /**
     * 显示插屏广告
     */
    platform.prototype.showInterstitial = function (onComplete) {
        if (this.initialized_) {
            this.onSuccessedCallback_ = this.onFailureCallback_ = function () {
                window.focus();
                onComplete && onComplete();
                onComplete = null;
            };
            window["gdsdk"].showAd();
        }
        else {
            onComplete && onComplete();
            onComplete = null;
        }
    };
    platform.prototype.showReward = function (onSuccessed, onFailure) {
        // onSuccessed && onSuccessed();
        // onSuccessed = null;
        // return;

        if (this.initialized_) {
            this.onSuccessedCallback_ = function () {
                window.focus();
                onSuccessed && onSuccessed();
                onSuccessed = null;
            };
            this.onFailureCallback_ = function () {
                window.focus();
                onFailure && onFailure();
                onFailure = null;
            };
            window["gdsdk"].showAd('rewarded');
        }
        else {
            if(onFailure){
                onFailure();
                onFailure = null;
            }else{
                this.prompt("Failed to get the reward, please watch the ads to the end.");
            }
        }
    };

    platform.prototype.prompt = function(msg) {
        if (!this.prompt_) {
            this.prompt_ = document.createElement("div");
            this.prompt_.style.cssText= "font-family:siyuan;max-width:80%;min-width: 320px;padding:10px 10px 10px 10px;min-height: 40px;color: rgb(255, 255, 255);line-height: 20px;text-align: center;border-radius: 4px;position: fixed;top: 40%;left: 50%;transform: translate(-50%, -50%);z-index: 999999;background: rgba(0, 0, 0,.7);font-size: 16px;";
            document.body.appendChild(this.prompt_);
        }
        this.prompt_.innerHTML =msg;
        this.prompt_.style.display = "inline";
        this.prompt_.style.opacity = 1
        setTimeout(function () {
            var d = 0.5;
            this.prompt_.style.webkitTransition = '-webkit-transform ' + d + 's ease-in, opacity ' + d + 's ease-in';
            this.prompt_.style.opacity = 0;
            this.prompt_.style.display = "none";
        }.bind(this), 1.5e3);
    }


    platform.prototype.getStorageSync = function(key) {
        let v = Laya.LocalStorage.getItem(key);
        return JSON.parse(v);
    };
    platform.prototype.setStorageSync = function(key, value) {
        return Laya.LocalStorage.setItem(key, JSON.stringify(value));
    };



    platform.getInstance = function () {
        if (!this.instance_) {
            this.instance_ = new platform();
        }
        return this.instance_;
    };


    platform.prototype.onEvent = function (event) {
        console.log("event.name ====", event);
        switch (event.name) {
            case "AD_ERROR":
                console.log("===============================AD_ERROR====================================");
                console.log("ErrorMessage:", event.name);
                console.log("ErrorType:", event.message);
                console.log("===========================================================================");
                this.event(ADEvent.AD_ERROR);
                this.onFailureCallback_ && this.onFailureCallback_(ADEvent.AD_ERROR);
                this.onFailureCallback_ = null;
                break;
            case "SDK_GAME_START":
                setTimeout(function () {
                    window["gdsdk"].preloadAd('rewarded')
                        .then(function (response) {
                        console.log("preloadAd_ok");
                    })
                        .catch(function (error) {
                        console.log('preload failed', error);
                        "";
                    });
                }, 200);
                if (event.status === "warning") {
                    this.event(ADEvent.AD_REQUEST_TOO_SOON);
                    this.onFailureCallback_ && this.onFailureCallback_(ADEvent.AD_REQUEST_TOO_SOON);
                    this.onFailureCallback_ = null;
                }
                break;
            case "SDK_GAME_PAUSE":
                break;
            case "LOADED":
                this.event(ADEvent.AD_LOADED);
                break;
            case "AD_SDK_FINISHED":
            case "CONTENT_RESUME_REQUESTED":
                this.event(ADEvent.AD_COMPLETE);
                this.onSuccessedCallback_ && this.onSuccessedCallback_(ADEvent.AD_COMPLETE);
                this.onSuccessedCallback_ = null;
                break;
            case "SDK_READY":
                window["gdsdk"].preloadAd('rewarded')
                    .then(function (response) {
                    console.log("preloadAd_ok");
                })
                    .catch(function (error) {
                    console.log('preload failed', error);
                    "";
                });
                break;
            case "AD_SDK_CANCEL":
            case "AD_CANCELED":
                this.event(ADEvent.AD_SKIPPED);
                this.onFailureCallback_ && this.onFailureCallback_(ADEvent.AD_REQUEST_TOO_SOON);
                this.onFailureCallback_ = null;
                break;
            default:
                break;
        }
    };
    return platform;
}(EventDispatcher));
